package meerkat.bulletinboard;

import com.google.protobuf.Message;

import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * Created by Arbel Deutsch Peled on 09-Dec-15.
 *
 * This class specifies the job that is required of a Bulletin Board Client Worker
 */
public class BulletinClientJob {

    public static enum JobType{
        POST_MESSAGE,       // Post a message to servers
        READ_MESSAGES,      // Read messages according to some given filter (any server will do)
        GET_REDUNDANCY    // Check the redundancy of a specific message in the databases
    }

    private List<String> serverAddresses;

    private int minServers;         // The minimal number of servers the job must be successful on for the job to be completed

    private final JobType jobType;

    private final Message payload;  // The information associated with the job type

    private int maxRetry;           // Number of retries for this job; set to -1 for infinite retries

    public BulletinClientJob(List<String> serverAddresses, int minServers, JobType jobType, Message payload, int maxRetry) {
        this.serverAddresses = serverAddresses;
        this.minServers = minServers;
        this.jobType = jobType;
        this.payload = payload;
        this.maxRetry = maxRetry;
    }

    public void updateServerAddresses(List<String> newServerAdresses) {
        this.serverAddresses = newServerAdresses;
    }

    public List<String> getServerAddresses() {
        return serverAddresses;
    }

    public int getMinServers() {
        return minServers;
    }

    public JobType getJobType() {
        return jobType;
    }

    public Message getPayload() {
        return payload;
    }

    public int getMaxRetry() {
        return maxRetry;
    }

    public void shuffleAddresses() {
        Collections.shuffle(serverAddresses);
    }

    public void decMinServers(){
        minServers--;
    }

    public void decMaxRetry(){
        if (maxRetry > 0) {
            maxRetry--;
        }
    }

    public boolean isRetry(){
        return (maxRetry != 0);
    }

}