package meerkat.bulletinboard.callbacks;

import com.google.common.util.concurrent.ListeningExecutorService;
import meerkat.bulletinboard.BulletinBoardClient;
import meerkat.bulletinboard.BulletinClientJobResult;
import meerkat.protobuf.BulletinBoardAPI.*;

import java.util.List;

/**
 * This is a future callback used to listen to workers and run on job finish
 * Depending on the type of job and the finishing status of the worker: a decision is made whether to retry or return an error
 */
public class GetRedundancyFutureCallback extends ClientFutureCallback {

    private BulletinBoardClient.ClientCallback<Float> callback;

    public GetRedundancyFutureCallback(ListeningExecutorService listeningExecutor,
                                BulletinBoardClient.ClientCallback<Float> callback) {
        super(listeningExecutor);
        this.callback = callback;
    }

    @Override
    public void onSuccess(BulletinClientJobResult result) {

        int absoluteRedundancy = ((IntMsg) result.getResult()).getValue();
        int totalServers = result.getJob().getServerAddresses().size();

        callback.handleCallback( ((float) absoluteRedundancy) / ((float) totalServers) );

    }

    @Override
    public void onFailure(Throwable t) {
        callback.handleFailure(t);
    }
}
