package meerkat.bulletinboard.callbacks;

import com.google.common.util.concurrent.Futures;
import com.google.common.util.concurrent.ListeningExecutorService;
import meerkat.bulletinboard.BulletinBoardClient;
import meerkat.bulletinboard.BulletinClientJob;
import meerkat.bulletinboard.BulletinClientJobResult;
import meerkat.bulletinboard.BulletinClientWorker;
import meerkat.protobuf.BulletinBoardAPI;

import java.util.List;

/**
 * This is a future callback used to listen to workers and run on job finish
 * Depending on the type of job and the finishing status of the worker: a decision is made whether to retry or return an error
 */
public class ReadMessagesFutureCallback extends ClientFutureCallback {

    private BulletinBoardClient.ClientCallback<List<BulletinBoardAPI.BulletinBoardMessage>> callback;

    public ReadMessagesFutureCallback(ListeningExecutorService listeningExecutor,
                                      BulletinBoardClient.ClientCallback<List<BulletinBoardAPI.BulletinBoardMessage>> callback) {
        super(listeningExecutor);
        this.callback = callback;
    }

    @Override
    public void onSuccess(BulletinClientJobResult result) {

        callback.handleCallback(((BulletinBoardAPI.BulletinBoardMessageList) result.getResult()).getMessageList());

    }

    @Override
    public void onFailure(Throwable t) {
        callback.handleFailure(t);
    }
}
