package meerkat.bulletinboard.sqlserver;

import meerkat.protobuf.BulletinBoardAPI.*;
import org.sqlite.SQLiteDataSource;

import javax.sql.DataSource;
import java.util.LinkedList;
import java.util.List;

/**
 * Created by Arbel Deutsch Peled on 09-Dec-15.
 */

public class SQLiteQueryProvider implements BulletinBoardSQLServer.SQLQueryProvider {

    String dbName;

    public SQLiteQueryProvider(String dbName) {
        this.dbName = dbName;
    }

    @Override
    public String getSQLString(QueryType queryType) throws IllegalArgumentException{

        switch(queryType) {
            case ADD_SIGNATURE:
                return "INSERT OR IGNORE INTO SignatureTable (EntryNum, SignerId, Signature) VALUES (:EntryNum,:SignerId,:Signature)";
            case CONNECT_TAG:
                return "INSERT OR IGNORE INTO MsgTagTable (TagId, EntryNum)"
                    + " SELECT TagTable.TagId, :EntryNum AS EntryNum FROM TagTable WHERE Tag = :Tag";
            case FIND_MSG_ID:
                return "SELECT EntryNum From MsgTable WHERE MsgId = :MsgId";
            case GET_MESSAGES:
                return "SELECT MsgTable.EntryNum, MsgTable.Msg FROM MsgTable";
            case GET_SIGNATURES:
                return "SELECT Signature FROM SignatureTable WHERE EntryNum = :EntryNum";
            case INSERT_MSG:
                return "INSERT INTO MsgTable (MsgId, Msg) VALUES(:MsgId,:Msg)";
            case INSERT_NEW_TAG:
                return "INSERT OR IGNORE INTO TagTable(Tag) VALUES (:Tag)";
           default:
               throw new IllegalArgumentException("Cannot serve a query of type " + queryType);
        }

    }

    @Override
    public String getCondition(FilterType filterType, int serialNum) throws IllegalArgumentException {

        String serialString = Integer.toString(serialNum);

        switch(filterType) {
            case EXACT_ENTRY:
                return "MsgTable.EntryNum = :EntryNum" + serialString;
            case MAX_ENTRY:
                return "MsgTable.EntryNum <= :EntryNum" + serialString;
            case MAX_MESSAGES:
                return "LIMIT = :Limit" + serialString;
            case MSG_ID:
                return "MsgTable.MsgId = :MsgId" + serialString;
            case SIGNER_ID:
                return "EXISTS (SELECT 1 FROM SignatureTable"
                        + " WHERE SignatureTable.SignerId = :SignerId" + serialString + " AND SignatureTable.EntryNum = MsgTable.EntryNum)";
            case TAG:
                return "EXISTS (SELECT 1 FROM TagTable"
                        + " INNER JOIN MsgTagTable ON TagTable.TagId = MsgTagTable.TagId"
                        + " WHERE TagTable.Tag = :Tag" + serialString + " AND MsgTagTable.EntryNum = MsgTable.EntryNum)";
            default:
                throw new IllegalArgumentException("Cannot serve a filter of type " + filterType);
        }

    }

    @Override
    public String getConditionParamTypeName(FilterType filterType) throws IllegalArgumentException {
        return null; //TODO: write this.
    }

    @Override
    public DataSource getDataSource() {
        // TODO: Fix this
        SQLiteDataSource dataSource = new SQLiteDataSource();
        dataSource.setUrl("jdbc:sqlite:" + dbName);
        dataSource.setDatabaseName("meerkat"); //TODO: Make generic

        return dataSource;
    }


    @Override
    public List<String> getSchemaCreationCommands() {
        List<String> list = new LinkedList<String>();

        list.add("CREATE TABLE IF NOT EXISTS MsgTable (EntryNum INTEGER PRIMARY KEY, MsgId BLOB UNIQUE, Msg BLOB)");

        list.add("CREATE TABLE IF NOT EXISTS TagTable (TagId INTEGER PRIMARY KEY, Tag varchar(50) UNIQUE)");
        list.add("CREATE TABLE IF NOT EXISTS MsgTagTable (EntryNum BLOB, TagId INTEGER, FOREIGN KEY (EntryNum)"
                + " REFERENCES MsgTable(EntryNum), FOREIGN KEY (TagId) REFERENCES TagTable(TagId), UNIQUE (EntryNum, TagID))");

        list.add("CREATE TABLE IF NOT EXISTS SignatureTable (EntryNum INTEGER, SignerId BLOB, Signature BLOB,"
                + " FOREIGN KEY (EntryNum) REFERENCES MsgTable(EntryNum))");

        list.add("CREATE INDEX IF NOT EXISTS SignerIndex ON SignatureTable(SignerId)");
        list.add("CREATE UNIQUE INDEX IF NOT EXISTS SignerIndex ON SignatureTable(SignerId, EntryNum)");

        return list;
    }

    @Override
    public List<String> getSchemaDeletionCommands() {
        List<String> list = new LinkedList<String>();

        list.add("DROP TABLE IF EXISTS MsgTagTable");

        list.add("DROP INDEX IF EXISTS SignerIndex");
        list.add("DROP TABLE IF EXISTS SignatureTable");

        list.add("DROP TABLE IF EXISTS TagTable");
        list.add("DROP TABLE IF EXISTS MsgTable");

        return list;
    }
}
