package meerkat.bulletinboard.webapp;

import javax.servlet.ServletContext;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;

import meerkat.bulletinboard.BulletinBoardServer;
import meerkat.bulletinboard.sqlserver.BulletinBoardSQLServer;
import meerkat.bulletinboard.sqlserver.H2QueryProvider;
import meerkat.bulletinboard.sqlserver.MySQLQueryProvider;
import meerkat.bulletinboard.sqlserver.SQLiteQueryProvider;
import meerkat.comm.CommunicationException;
import meerkat.protobuf.BulletinBoardAPI.BoolMsg;
import meerkat.protobuf.BulletinBoardAPI.BulletinBoardMessage;
import meerkat.protobuf.BulletinBoardAPI.BulletinBoardMessageList;
import meerkat.protobuf.BulletinBoardAPI.MessageFilterList;
import meerkat.rest.Constants;

@Path(Constants.BULLETIN_BOARD_SERVER_PATH)
public class BulletinBoardWebApp implements BulletinBoardServer, ServletContextListener{
		
	private static final String BULLETIN_BOARD_ATTRIBUTE_NAME = "bulletinBoard";
	
	@Context ServletContext servletContext;
	
	BulletinBoardServer bulletinBoard;
	
	/**
	 * This is the servlet init method.
	 */
	public void init(){
		bulletinBoard = (BulletinBoardServer) servletContext.getAttribute(BULLETIN_BOARD_ATTRIBUTE_NAME);
	}
	
	/**
	 * This is the BulletinBoard init method.
	 */
	@Override
	public void init(String meerkatDB) throws CommunicationException {
		bulletinBoard.init(meerkatDB);
	}
	
	@Override
	public void contextInitialized(ServletContextEvent servletContextEvent) {
		ServletContext servletContext = servletContextEvent.getServletContext();
		String dbType = servletContext.getInitParameter("dbType");
		String dbName = servletContext.getInitParameter("dbName");

		if ("SQLite".equals(dbType)){

			bulletinBoard = new BulletinBoardSQLServer(new SQLiteQueryProvider(dbName));

		} else if ("H2".equals(dbType)) {

			bulletinBoard = new BulletinBoardSQLServer(new H2QueryProvider(dbName));

		} else if ("MySQL".equals(dbType)) {

			String dbAddress = servletContext.getInitParameter("dbAddress");
			int dbPort = Integer.parseInt(servletContext.getInitParameter("dbPort"));
			String username = servletContext.getInitParameter("username");
			String password = servletContext.getInitParameter("password");

			bulletinBoard = new BulletinBoardSQLServer(new MySQLQueryProvider(dbAddress,dbPort,dbName,username,password));
		}

		try {
			init(dbName);
			servletContext.setAttribute(BULLETIN_BOARD_ATTRIBUTE_NAME, bulletinBoard);
		} catch (CommunicationException e) {
			System.err.println(e.getMessage());
		}
	}
	
	@Path(Constants.POST_MESSAGE_PATH)
	@POST
	@Consumes(Constants.MEDIATYPE_PROTOBUF)
	@Produces(Constants.MEDIATYPE_PROTOBUF)
	@Override
	public BoolMsg postMessage(BulletinBoardMessage msg) throws CommunicationException {
		init();
		return bulletinBoard.postMessage(msg);
	}
	
	@Path(Constants.READ_MESSAGES_PATH)
	@POST
	@Consumes(Constants.MEDIATYPE_PROTOBUF)
	@Produces(Constants.MEDIATYPE_PROTOBUF)
	@Override
	public BulletinBoardMessageList readMessages(MessageFilterList filterList) throws CommunicationException {
		init();
		return bulletinBoard.readMessages(filterList);
	}

	@Override
	public void close(){
		try {
			bulletinBoard.close();
		} catch (CommunicationException e) {
			System.err.println(e.getMessage());
		}
	}
	
	@GET
    @Produces(MediaType.TEXT_PLAIN)
	public String test() {
        return "This BulletinBoard is up and running!\n Please consult the API documents to perform queries.";
	}
	
	@Override
	public void contextDestroyed(ServletContextEvent servletContextEvent) {
		ServletContext servletContext = servletContextEvent.getServletContext();
		bulletinBoard = (BulletinBoardServer) servletContext.getAttribute(BULLETIN_BOARD_ATTRIBUTE_NAME);
		close();
	}
	
}
