package SecureDistributedKeyGenerationForDiscreteLogBasedCryptosystem;

import Communication.User;
import FeldmanVerifiableSecretSharing.VerifiableSecretSharing;
import JointFeldmanProtocol.DistributedKeyGeneration;
import ShamirSecretSharing.Polynomial;
import com.google.protobuf.ByteString;
import meerkat.protobuf.DKGMessages;
import org.factcenter.qilin.primitives.Group;

import java.math.BigInteger;
import java.util.Random;
import java.util.Set;

/**
 * Created by Tzlil on 3/16/2016.
 * TODO: comments
 * TODO: put Channel (User) in constructor
 */
public class SecureDistributedKeyGeneration extends DistributedKeyGeneration {

    private VerifiableSecretSharing maskingShares;
    private final BigInteger h;
    private SecureDistributedKeyGenerationParty[] parties;

    public SecureDistributedKeyGeneration(int t, int n, BigInteger zi, Random random, BigInteger q, BigInteger g
            , BigInteger h, Group<BigInteger> group, int id) {
        super(t, n, zi, random, q, g, group, id);
        this.h = h;
        BigInteger r = new BigInteger(q.bitLength(),random).mod(q);
        this.maskingShares = new VerifiableSecretSharing(t,n,r,random,q,h,group);
        this.parties = new SecureDistributedKeyGenerationParty[n];
        for (int i = 1; i <= n ; i++){
            this.parties[i - 1] = new SecureDistributedKeyGenerationParty(i,n,t);
        }
        this.parties[id - 1].share = getShare(id);
        this.parties[id - 1].shareT = maskingShares.getShare(id);
        super.setParties(parties);
    }

    protected SecureDistributedKeyGenerationParty[] getParties(){
        return parties;
    }

    protected void setParties(SecureDistributedKeyGenerationParty[] parties) {
        super.setParties(parties);
        this.parties = parties;
    }


    @Override
    public void sendSecret(User user, int j) {
        Polynomial.Point secret = getShare(j);
        Polynomial.Point secretT = maskingShares.getShare(j);
        DKGMessages.DoubleSecretMessage doubleSecretMessage = doubleShareMessage(id,j,secret,secretT);
        // TODO: Change SECRET to SHARE
        user.send(j, DKGMessages.Mail.Type.SECRET, doubleSecretMessage);
    }


    public boolean isValidShare(int i){
        SecureDistributedKeyGenerationParty party = parties[i - 1];
        return isValidShare(party.share, party.shareT, party.verifiableValues, id);
    }

    /**
     * TODO: comment
     * @param share
     * @param shareT
     * @param verificationValues
     * @param j
     * @return computeVerificationValue(j,verificationValues,group) == (g ^ share.y) * (h ^ shareT.y) mod q
     */
    public boolean isValidShare(Polynomial.Point share, Polynomial.Point shareT, BigInteger[] verificationValues, int j){
        try {
            BigInteger v = computeVerificationValue(j, verificationValues, group);
            BigInteger exp = group.add(group.multiply(g, share.y), group.multiply(h, shareT.y));
            return exp.equals(v);
        }
        catch (NullPointerException e){
            return false;
        }
    }

    // TODO: comment
    private void broadcastComplaint(User user,Polynomial.Point share,Polynomial.Point shareT,int i){
        DKGMessages.DoubleSecretMessage complaint = doubleShareMessage(i,id,share,shareT);
        user.broadcast(DKGMessages.Mail.Type.COMPLAINT,complaint);
    }

    /**
     * stage4.3 according to the protocol
     *  if check fails for index i, Pj
     */
    public void computeAndBroadcastComplaints(User user, Set<Integer> QUAL){
        SecureDistributedKeyGenerationParty party;
        for (int i : QUAL) {
            party = parties[i - 1];
            if (i != id) {
                if (!super.isValidSecret(party.share, party.commitments, id)) {
                    broadcastComplaint(user, party.share, party.shareT, i);
                }
            }
        }
    }

    public void broadcastVerificationValues(User user){
        BigInteger[] verificationValues = new BigInteger[t + 1];
        BigInteger[] hBaseCommitments = maskingShares.getCommitmentsArray();
        for (int k = 0 ; k < verificationValues.length ; k++){
            verificationValues[k] = group.add(commitmentsArray[k],hBaseCommitments[k]);
        }
        broadcastCommitments(user,verificationValues);
    }

    private DKGMessages.DoubleSecretMessage doubleShareMessage(int i, int j, Polynomial.Point secret, Polynomial.Point secretT){
        DKGMessages.DoubleSecretMessage doubleSecretMessage = DKGMessages.DoubleSecretMessage.newBuilder()
                .setI(i)
                .setJ(j)
                .setSecret(ByteString.copyFrom(secret.y.toByteArray()))
                .setSecretT(ByteString.copyFrom(secretT.y.toByteArray()))
                .build();
        return doubleSecretMessage;
    }

    @Override
    public void broadcastComplaintAnswer(User user, int j) {
        DKGMessages.DoubleSecretMessage answer = doubleShareMessage(id,j,getShare(j)
                , maskingShares.getShare(j));
        user.broadcast(DKGMessages.Mail.Type.ANSWER,answer);
    }

    public void broadcastAnswer(User user,Polynomial.Point secret,Polynomial.Point secretT,int i){
        DKGMessages.DoubleSecretMessage complaint = doubleShareMessage(i,id,secret,secretT);
        user.broadcast(DKGMessages.Mail.Type.ANSWER,complaint);
    }

    public BigInteger getH() {
        return h;
    }
}
