package SecureDistributedKeyGenerationForDiscreteLogBasedCryptosystem;

import Communication.MailHandler;
import Communication.MessageHandler;
import com.google.protobuf.InvalidProtocolBufferException;
import com.google.protobuf.Message;
import meerkat.protobuf.DKGMessages;

/**
 * Created by Tzlil on 2/29/2016.
 */
public class SecureDistributedKeyGenerationMailHandler extends MailHandler {

    private boolean isStage4;

    public SecureDistributedKeyGenerationMailHandler(MessageHandler messageHandler) {
        super(messageHandler);
        this.isStage4 = false;
    }

    @Override
    public Message extractMessage(DKGMessages.Mail mail) {
        try {
            Message message;
            switch (mail.getType()) {
                case SECRET:
                    message = DKGMessages.DoubleSecretMessage.parseFrom(mail.getMessage());
                    break;
                case COMMITMENT:
                    message = DKGMessages.CommitmentMessage.parseFrom(mail.getMessage());
                    break;
                case COMPLAINT:
                    if(!isStage4)
                        message = DKGMessages.IDMessage.parseFrom(mail.getMessage());
                    else
                        message = DKGMessages.DoubleSecretMessage.parseFrom(mail.getMessage());
                    break;
                case DONE:
                    message = DKGMessages.EmptyMessage.parseFrom(mail.getMessage());
                    break;
                case ANSWER:
                    message = DKGMessages.DoubleSecretMessage.parseFrom(mail.getMessage());
                    break;
                case ABORT:
                    message = DKGMessages.EmptyMessage.parseFrom(mail.getMessage());
                    break;
                default:
                    return null;
            }
            return message;
        } catch (InvalidProtocolBufferException e) {
            return null;
        }
    }

    public boolean isStage4() {
        return isStage4;
    }

    public void setStage4(boolean stage4) {
        isStage4 = stage4;
    }
}
