package JointFeldmanProtocol;

import Communication.MailHandler;
import Communication.Network;

import java.math.BigInteger;
import java.util.*;

/**
 * Created by Tzlil on 3/21/2016.
 */
public class DKGMaliciousUserImpl extends DistributedKeyGenerationUserImpl {

    private final DistributedKeyGeneration maliciousDkg;
    private final Set<Integer> falls;
    public DKGMaliciousUserImpl(DistributedKeyGeneration dkg,DistributedKeyGeneration maliciousDKG, Network network,Set<Integer> falls) {
        super(dkg, network);
        this.falls = falls;
        this.maliciousDkg = maliciousDKG;
        maliciousDKG.setParties(parties);
    }

    public static Set<Integer> selectFallsRandomly(Set<Integer> ids, Random random){
        Set<Integer> falls = new HashSet<Integer>();
        ArrayList<Integer> idsList = new ArrayList<Integer>();
        for (int id : ids){
            idsList.add(id);
        }
        int fallsSize = random.nextInt(idsList.size()) + 1;// 1 - (n-1)
        while (falls.size() < fallsSize){
            falls.add(idsList.remove(random.nextInt(idsList.size())));
        }
        return falls;
    }

    public static DistributedKeyGeneration generateMaliciousDKG(DistributedKeyGeneration dkg,Random random){
        BigInteger q = dkg.getQ();
        BigInteger zi = new BigInteger(q.bitLength(), random).mod(q);
        return new DistributedKeyGeneration(dkg.getT(),dkg.getN(),zi,random,dkg.getQ()
                ,dkg.getGenerator(),dkg.getGroup(),dkg.getId());
    }

    @Override
    public void stage1() {
        dkg.broadcastCommitments(user);
        sendSecrets(); //insteadof dkg.sendSecrets(user);
    }

    @Override
    public void stage3() {
        maliciousDkg.answerAllComplainingPlayers(user);
    }

    @Override
    public void stage4(){
        // do nothing
    }

    private void sendSecrets(){
        for (int j = 1; j <= n ; j++){
            if(j != id){
                if(falls.contains(j)){
                    maliciousDkg.sendSecret(user,j);
                }else {
                    dkg.sendSecret(user, j);
                }
            }
        }
    }


}
