package meerkat.bulletinboard;

import meerkat.comm.*;
import meerkat.protobuf.Voting.*;

import static meerkat.protobuf.BulletinBoardAPI.*;

import java.util.List;

/**
 * Created by talm on 24/10/15.
 */
public interface BulletinBoardClient {

    interface ClientCallback<T> {
        void handleCallback(T msg);
        void handleFailure(Throwable t);
    }

    /**
     * Initialize the client to use some specified servers
     * @param clientParams contains the parameters required for the client setup
     */
    void init(BulletinBoardClientParams clientParams);

    /**
     * Post a message to the bulletin board
     * @param msg
     */
    MessageID postMessage(BulletinBoardMessage msg, ClientCallback<?> callback);

    /**
     * Check how "safe" a given message is
     * @param id
     * @return a normalized "redundancy score" from 0 (local only) to 1 (fully published)
     */
    void getRedundancy(MessageID id, ClientCallback<Float> callback);

    /**
     * Read all messages posted matching the given filter
     * Note that if messages haven't been "fully posted", this might return a different
     * set of messages in different calls. However, messages that are fully posted
     * are guaranteed to be included.
     * @param filterList return only messages that match the filters (null means no filtering).
     */
    void readMessages(MessageFilterList filterList, ClientCallback<List<BulletinBoardMessage>> callback);

    /**
     * Closes all connections, if any.
     * This is done in a synchronous (blocking) way.
     */
    void close();

}
