package meerkat.util;

import meerkat.protobuf.BulletinBoardAPI;
import meerkat.protobuf.BulletinBoardAPI.*;
import meerkat.protobuf.Crypto.*;

import java.util.Comparator;
import java.util.List;

/**
 * Created by Arbel Deutsch Peled on 05-Dec-15.
 * This class implements a comparison between BulletinBoardMessage instances that disregards:
 * 1. The entry number (since this can be different between database instances)
 * 2. The order of the signatures
 */
public class BulletinBoardMessageComparator implements Comparator<BulletinBoardMessage> {

    /**
     * Compare the messages
     * @param msg1
     * @param msg2
     * @return 0 if the messages are equivalent (see above) and -1 otherwise.
     */
    @Override
    public int compare(BulletinBoardMessage msg1, BulletinBoardMessage msg2) {

        List<Signature> msg1Sigs = msg1.getSigList();
        List<Signature> msg2Sigs = msg2.getSigList();

        // Compare unsigned message
        if (!msg1.getMsg().equals(msg2.getMsg())){
            return -1;
        }

        // Compare signatures

        if (msg1Sigs.size() != msg2Sigs.size()){
            return -1;
        }

        for (Signature sig : msg1Sigs){
            if (!msg2Sigs.contains(sig)) {
                return -1;
            }
        }

        return 0;
    }
}
